#!/usr/bin/perl -I/usr/sausalito/perl
#
# $Id: generate_graphs.pl
#
# This handler polls the RRD databases for the master node and VPS network traffic
# statistics and then generates the respective traffic graphics.
#

use CCE;
use POSIX qw(isalpha);
use RRDs;
use Sys::Hostname;
use File::Spec;

$host = hostname();
$cce = new CCE('Domain' => 'base-traffic');
$cce->connectfd();

$img = '/usr/sausalito/ui/web/base/traffic/images';
$rrd = '/var/lib/averrd';

opendir (DIR,"$rrd");
while ($datei = readdir(DIR)) {
        if ($datei =~ /\.rrd/) {
                @mgraphs=split(/\./,$datei);
                push (@graphs, $mgraphs[0]);
        }
}
closedir(DIR);

@sorted_graphs = sort { lc($a) cmp lc($b) } (@graphs);
foreach $line (@sorted_graphs) {

        $hostname = "";
        @vpsid_arr = split(/_/,$line);
        $vpsid = $vpsid_arr['1'];
        if ($vpsid) {
                $hostname = `/usr/sbin/vzlist -a -H -o veid,hostname |grep $vpsid | awk '{print \$2}'`;
                chomp($hostname);
        }

        if ($hostname ne "") {
                &CreateGraph($line, "day", $hostname);
                &CreateGraph($line, "week", $hostname);
                &CreateGraph($line, "month", $hostname);
                &CreateGraph($line, "year", $hostname);
        }
        else {
                &CreateGraph($line, "day", $host);
                &CreateGraph($line, "week", $host);
                &CreateGraph($line, "month", $host);
                &CreateGraph($line, "year", $host);
        }
}

sub CreateGraph {
        # creates graph
        # inputs: $_[0]: interface name (ie, eth0/eth1/eth2/ppp0)
        #         $_[1]: interval (ie, day, week, month, year)
        #         $_[2]: interface description

        RRDs::graph "$img/$_[0]-$_[1].png",
                "-s -1$_[1]",
                "-t traffic on $_[0] :: $_[2]",
                "--lazy",
                "-h", "80", "-w", "640",
                "-l 0",
                "-a", "PNG",
                "-v bytes/sec",
                "DEF:in=$rrd/$_[0].rrd:in:AVERAGE",
                "DEF:out=$rrd/$_[0].rrd:out:AVERAGE",
                "CDEF:out_neg=out,-1,*",
                "AREA:in#32CD32:Incoming",
                "LINE1:in#336600",
                "GPRINT:in:MAX:  Max\\: %5.1lf %s",
                "GPRINT:in:AVERAGE: Avg\\: %5.1lf %S",
                "GPRINT:in:LAST: Current\\: %5.1lf %Sbytes/sec\\n",
                "AREA:out_neg#4169E1:Outgoing",
                "LINE1:out_neg#0033CC",
                "GPRINT:out:MAX:  Max\\: %5.1lf %S",
                "GPRINT:out:AVERAGE: Avg\\: %5.1lf %S",
                "GPRINT:out:LAST: Current\\: %5.1lf %Sbytes/sec",
                "HRULE:0#000000";
        #if ($ERROR = RRDs::error) { print "$0: unable to generate $_[0] $_[1] traffic graph: $ERROR\n"; }

        # Fix permissions:
        system("/bin/chmod 644 $img/$_[0]-$_[1].png");
}

$cce->bye('SUCCESS');
exit(0);

# 
# Copyright (c) 2016 Michael Stauber, SOLARSPEED.NET
# All Rights Reserved.
# 
# 1. Redistributions of source code must retain the above copyright 
#         notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright 
#         notice, this list of conditions and the following disclaimer in 
#         the documentation and/or other materials provided with the 
#         distribution.
# 
# 3. Neither the name of the copyright holder nor the names of its 
#         contributors may be used to endorse or promote products derived 
#         from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.
# 
# You acknowledge that this software is not designed or intended for 
# use in the design, construction, operation or maintenance of any 
# nuclear facility.
# 